/*
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with this program;
 * if not, see <http://www.gnu.org/licenses/>.
*/
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "soundsettingsdlg.h"

#include <math.h>
#include <iostream>

#include "static_settings.h"
#include "fftprocessor.h"

#include <QMessageBox>

using namespace std;

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::MainWindowClass)
{
    printf("Setting up user interface...\n");
    flush(cout);

    ui->setupUi(this);

    ap = new audioProcessor(DEFAULT_FFT_SIZE);
    ap->setFrameLength(DEFAULT_FRAME_LENGTH);

    mainPlot = new QwtPlot();
    initPlot(mainPlot, "Frequency", "Magnitude", 0, ap->getSampleRate()/2, -120, 100);
    mainCurveZoomer = new QwtPlotZoomer(mainPlot->canvas(),true);
    mainCurveZoomer->setMousePattern(QwtEventPattern::MouseSelect2, Qt::RightButton, Qt::ControlModifier);
    mainCurveZoomer->setMousePattern(QwtEventPattern::MouseSelect3,Qt::RightButton);

    ui->verticalLayout->insertWidget(0,mainPlot);
    mainCurve = new QwtPlotCurve();
    mainCurve->attach(mainPlot);

    mainCureXData = (double*) malloc(sizeof(double) * DEFAULT_FFT_SIZE/2+1);
    mainCureYData = (double*) malloc(sizeof(double) * DEFAULT_FFT_SIZE/2+1);

    connect(ui->soundcardSettingsAction, SIGNAL(triggered()), this, SLOT(onSoundcardSettingsAction()));
    connect(ui->continuousRunPushButton, SIGNAL(pressed()), this, SLOT(onContinuousRunPushButton()));
    connect(ui->stopRunPushButton, SIGNAL(pressed()), this, SLOT(onStopRunPushButton()));
    connect(ap, SIGNAL(frameProcessed(fftProcessor*)), this, SLOT(onFrameProcessed(fftProcessor*)));

}

MainWindow::~MainWindow()
{
  delete mainPlot;
  delete ui;
  delete mainCurveZoomer;
  delete ap;
}

void MainWindow::initPlot(QwtPlot* plot, const char* xTitle, const char* yTitle, int xmin, int xmax, int ymin, int ymax)
{
    QwtText xQwTitle = QwtText(xTitle);
    QwtText yQwTitle = QwtText(yTitle);

    QwtText title = QwtText("QwtPlot");
    title.setFont(QFont("SansSerif", 8));

    QwtText xAxisTitle = xQwTitle;
    xAxisTitle.setFont(QFont("SansSerif", 8));
    QwtText yAxisTitle = yQwTitle;
    yAxisTitle.setFont(QFont("SansSerif", 8));

    plot->setAxisAutoScale(QwtPlot::yLeft);
    plot->setAxisAutoScale(QwtPlot::xBottom);

    QFont axisFont = QFont("SansSerif", 6);
    plot->setAxisFont(QwtPlot::xBottom, axisFont);
    plot->setAxisFont(QwtPlot::yLeft, axisFont);
    // Axis
    plot->setAxisTitle(QwtPlot::xBottom, xAxisTitle);
    plot->setAxisTitle(QwtPlot::yLeft, yAxisTitle);

    QwtPlotGrid *grid = new QwtPlotGrid();
    grid->setMajPen(QPen(Qt::gray, 0, Qt::DotLine));
    grid->setMinPen(QPen(Qt::gray, 0 , Qt::DotLine));
    grid->attach(plot);

    //	p.setAxisScale(QwtPlot::xBottom, 0, 2*3.14);
    plot->setCanvasBackground(QColor(255,255,255));
    plot->setGeometry(QRect(20, 20, 600, 220));
    plot->resize(200, 200);

    plot->setAxisScale(QwtPlot::xBottom, xmin, xmax);
    plot->setAxisScale(QwtPlot::yLeft, ymin, ymax);

    QwtPlotPanner *panner = new QwtPlotPanner(plot->canvas());
    panner->setMouseButton(Qt::MidButton);
}

void MainWindow::onSoundcardSettingsAction()
{
  soundSettingsDlg dlg(ap);
  dlg.exec();
}

void MainWindow::onContinuousRunPushButton()
{
  for(int i=0; i < DEFAULT_FFT_SIZE/2+1; i++)
    mainCureXData[i] = ((double) i) / (DEFAULT_FFT_SIZE/2+1) * ap->getSampleRate()/2;

  mainPlot->setAxisScale(QwtPlot::yLeft, -120, 10);
  mainPlot->setAxisScale(QwtPlot::xBottom, 0, ap->getSampleRate()/2);

  int err = ap->startAudio();
  if(err < 0)
  {
    QString message(ap->getErrorText(err));
    QMessageBox messageBox(QMessageBox::Critical, "Error Starting Audio", message);
    messageBox.exec();
  }
}

void MainWindow::onStopRunPushButton()
{
  ap->stopAudio();
}

void MainWindow::onFrameProcessed(fftProcessor *fft)
{

  unsigned int n=fft->getfftSize()/2+1;
  double one_over_n=1/((double) n); //to make it a little bit faster

  for(unsigned int i=0; i < n; i++)
  {
    //log+abs fft:
    mainCureYData[i] = 20.0 * log10(one_over_n * fft->fftOutAbs[i]);
    //abs fft:
//      mainCureYData[i] = sqrt(fft->fftOut[i][0]*fft->fftOut[i][0] + fft->fftOut[i][1]*fft->fftOut[i][1]);
    //real input:
//      mainCureYData[i] = fft->fftIn[i];
  }

  mainCurve->setData(mainCureXData, mainCureYData, fft->getfftSize()/2+1);
  mainPlot->replot();

}

